/*
CartoTypeBase.h
Copyright 2011-2023 CartoType Ltd.
See www.cartotype.com for more information.
*/

#ifndef CartoTypeBase_h__
#define CartoTypeBase_h__

#import <Foundation/Foundation.h>
#import <CoreGraphics/CGImage.h>
#import <CoreGraphics/CGGeometry.h>
#import <CartoType/CartoTypeResultCode.h>

/**
\file
Base types for use with the CartoType Objective C class CartoTypeFramework and others.
*/

/** Flags and constants to tell text searching how to match search terms with found strings. */
typedef enum
    {
    /** A flag to match all strings for which the search term is a perfect match or a prefix. */
    StringMatchPrefixFlag = 1,
    /** A flag to ignore all characters that are not letters, digits or whitespace when matching. */
    StringMatchIgnoreSymbolsFlag = 2,
    /** A flag to match accented and unaccented letters. */
    StringMatchFoldAccentsFlag = 4,
    /** A flag to allow imperfect matches with a small number of omitted, incorrect or extra characters. */
    StringMatchFuzzyFlag = 8,
    /** A flag to fold letter case. */
    StringMatchFoldCaseFlag = 16,
    /** A flag to ignore whitespace. */
    StringMatchIgnoreWhitespaceFlag = 32,
    
    /** Strings must match exactly. */
    StringMatchExact = 0,
    /** Fold case when matching strings. */
    StringMatchFoldCase = StringMatchFoldCaseFlag,
    /** The search term must be an exact match or a prefix of the found string. */
    StringMatchPrefix = StringMatchPrefixFlag,
    /** Ignore characters that are not letters, digits or whitespace.  */
    StringMatchIgnoreNonAlphanumerics = StringMatchIgnoreSymbolsFlag | StringMatchIgnoreWhitespaceFlag,
    /** Loose matching: ignore non-alphanumerics and fold accents and case. */
    StringMatchLoose = StringMatchIgnoreNonAlphanumerics | StringMatchFoldAccentsFlag | StringMatchFoldCaseFlag,
    /** Allow fuzzy matches: ignore non-alphanumerics, fold accents and allow imperfect matches. */
    StringMatchFuzzy = StringMatchIgnoreNonAlphanumerics | StringMatchFoldAccentsFlag | StringMatchFuzzyFlag | StringMatchFoldCaseFlag
    } CartoTypeStringMatchMethod;

/** File types of interest to CartoType. */
typedef enum
    {
    /** PNG (Portable Network Graphics) image files. */
    PngFileType,
    /** JPEG (Joint Photographic Expert Group) image files. */
    JpegFileType,
    /** TIFF (Tagged Image File Format) image files. */
    TiffFileType,
    /** CTM1 (CartoType Map Data Type 1) map data files. */
    Ctm1FileType,
    /** CTMS (CartoType Map Data, Serialized) map data files. */
    CtmsFileType,
    /** KML (Keyhole Markup Language) map data files. */
    KmlFileType,
    /** CTSQL (CartoType SQLite format) map data files. */
    CtsqlFileType,
    /** CTROUTE (CartoType XML route) files. */
    CartoTypeRouteFileType,
    /** GPX (GPS Exchange) files. */
    GpxFileType
    } CartoTypeFileType;

/** Map object types. */
typedef enum
    {
    /** A point in two-dimensional space. */
    PointMapObjectType,
    
    /** A line in two-dimensional space, consisting of one or more open contours. */
    LineMapObjectType,
    
    /** A polygon in two-dimensional space, consisting of one or more closed contours. */
    PolygonMapObjectType,
    
    /** Information arranged as an array of values, such as an image or height data. */
    ArrayMapObjectType,
    
    /** The total number of map object types. This is not an actual type. */
    MapObjectTypes,
    
    /** A value to use where a null type is needed. */
    NoMapObjectType = -1
    
    } CartoTypeMapObjectType;

/** The type of a CartoTypePathPoint. */
typedef enum
    {
    /** The point is on the curve. */
    OnCurvePointType = 0,
    /** The point is a control point for a quadratic (conic) Bezier spline. */
    QuadraticPointType = 1,
    /** The point is a control point for a cubic Bezier spline. */
    CubicPointType = 2
    } CartoTypePointType;

/** Alignments used for multi-line text. */
typedef enum
    {
    /** Center the text. */
    AlignCenter,
    /** Align to the left for left-to-right text, to the right for right-to-left text. */
    AlignStandard,
    /** Align to the right for left-to-right text, to the left for right-to-left text. */
    AlignReverse,
    /** Align to the left and leave space on the right. */
    AlignLeft,
    /** Align to the right and leave space on the left. */
    AlignRight
    } CartoTypeAlign;

/** A point on a path, defined using integer values. */
typedef struct
    {
    /** The X coordinate. */
    int32_t x;
    /** The Y coordinate. */
    int32_t y;
    /** The type: on the curve or a control point. */
    CartoTypePointType type;
    } CartoTypePathPoint;

/** A point on a path, defined using double-precision floating point values. */
typedef struct
    {
    /** The X coordinate. */
    double x;
    /** The Y coordinate. */
    double y;
    /** The type: on the curve or a control point. */
    CartoTypePointType type;
    } CartoTypePathPointFP;

/** A point defined using double-precision floating point values. */
typedef struct
    {
    /** The X coordinate. */
    double x;
    /** The Y coordinate. */
    double y;
    } CartoTypePoint;
 
/** An axis-aligned rectangle defined using double-precision floating point values. */
typedef struct
    {
    /** The minimum X coordinate: normally the left edge. */
    double minX;
    /** The minimum Y coordinate: the top if coordinates increase downwards, otherwise the bottom. */
    double minY;
    /** The maximum X coordinate: normally the right edge. */
    double maxX;
    /** The maximum Y coordinate: the bottom if coordinates increase downwards, otherwise the top. */
    double maxY;
    } CartoTypeRect;

/** A point and a result code. Used by functions that need to return a point but might not succeed. */
typedef struct
    {
    /** The point. */
    CartoTypePoint point;
    /** The result code.  */
    CTResult result;
    } CartoTypePointAndResult;
    
/** A rectangle and a result code. Used by functions that need to return a rectangle but might not succeed. */
typedef struct
    {
    /** The rectangle. */
    CartoTypeRect rect;
    /** The result code.  */
    CTResult result;
    } CartoTypeRectAndResult;
   
@class CartoTypeGeometry; // forward declaration

/** A geometry object and a result code. Used by functions that need to return a geometry object but might not succeed. */
@interface CartoTypeGeometryAndResult: NSObject
    /** The geometry object. */
    @property (nonatomic, strong) CartoTypeGeometry* geometry;
    /** The result code. */
    @property (nonatomic) CTResult result;
@end

/** An object ID and a result code. Used by functions that insert an object and need to return its identifier. */
typedef struct
    {
    /** The object ID.  */
    uint64_t objectId;
    /** The result code. */
    CTResult result;
    } CartoTypeIdAndResult;
    
/** An area, a length and a result code. */
typedef struct
    {
    /**The area in square meters.  */
    double area;
    /** The length or perimeter in meters. */
    double length;
    /** The result code. */
    CTResult result;
    } CartoTypeAreaLengthAndResult;

/** Coordinate types used when specifying points in many CartoType functions. */
typedef enum
    {
    /** Longitude (X) and latitude (Y) in degrees. X increases to the east and Y increases to the north. */
    DegreeCoordType,
    /** Pixels on the display: X increases to the right and Y increases downwards. */
    DisplayCoordType,
    /** A synonym for DisplayCoordType. Pixels on the display: X increases to the right and Y increases downwards. */
    ScreenCoordType = DisplayCoordType,
    /** Map coordinates: 32nds of projected meters. X increases to the east and Y increases to the north. */
    MapCoordType,
    /** Map meters: projected meters. X increases to the east and Y increases to the north. */
    MapMeterCoordType
    } CartoTypeCoordType;
    
/** Colors are represented as 32-bit integers holding 8 bits each for the read, green, blue and alpha values. */
typedef uint32_t CartoTypeColor;

enum
    {
    /** A value for the color black. */
    CTBlack = 0xFF000000,
    /** A value for the color dark gray. */
    CTDarkGray = 0xFF555555,
    /** A value for the color dark red. */
    CTDarkRed = 0xFF000080,
    /** A value for the color dark green. */
    CTDarkGreen = 0xFF008000,
    /** A value for the color dark yellow. */
    CTDarkYellow = 0xFF008080,
    /** A value for the color dark blue. */
    CTDarkBlue = 0xFF800000,
    /** A value for the color dark magenta. */
    CTDarkMagenta = 0xFF800080,
    /** A value for the color dark cyan. */
    CTDarkCyan = 0xFF808000,
    /** A value for the color red. */
    CTRed = 0xFF0000FF,
    /** A value for the color green. */
    CTGreen = 0xFF00FF00,
    /** A value for the color yellow. */
    CTYellow = 0xFF00FFFF,
    /** A value for the color blue. */
    CTBlue = 0xFFFF0000,
    /** A value for the color magenta. */
    CTMagenta = 0xFFFF00FF,
    /** A value for the color cyan. */
    CTCyan = 0xFFFFFF00,
    /** A value for the color gray. */
    CTGray = 0xFFAAAAAA,
    /** A value for the color white. */
    CTWhite = 0xFFFFFFFF,

    /** A value for the 'null color' transparent black. */
    CTTransparentBlack = 0x00000000
    };
    
/**
A rule to modify a certain style or styles.
The colors are blended with colors in a style, using their alpha values as the blend fraction.
*/
@interface CartoTypeBlendStyle: NSObject
    /**
    A space or comma separated list of wild-card style names.
    The special name '[legend]' is used for the legend, scale, copyright notice and turn instructions.
    */
    @property (nonatomic) NSString* styles;
    /** The color to be blended with the colors in the style. */
    @property (nonatomic) CartoTypeColor mainColor;
    /** If not null, the color to be blended with border colors. */
    @property (nonatomic) CartoTypeColor borderColor;
    /** If not null, the color to be blended with text colors. */
    @property (nonatomic) CartoTypeColor textColor;
    /** If not null, the color to be blended with text glow colors. */
    @property (nonatomic) CartoTypeColor textGlowColor;
    /** If not null, the forced color for icons. */
    @property (nonatomic) CartoTypeColor iconColor;
    
    /** Initializes a CartoTypeBlendStyle with a C++ blend style: for internal use only. */
    -(id)initWithBlendStyle:(void*)aBlendStyle;
    /** Returns a pointer to the C++ blend style: for internal use only. **/
    -(const void*)blendStyle;
@end

/** Parameters for the function to find an address. */
@interface CartoTypeAddress: NSObject
    /** The name or number of the building. */
    @property (nonatomic, strong) NSString* building;
    /** The name of a feature or place of interest. */
    @property (nonatomic, strong) NSString* feature;
    /** The street, road or other highway. */
    @property (nonatomic, strong) NSString* street;
    /** The suburb, neighborhood, quarter or other subdivision of the locality. */
    @property (nonatomic, strong) NSString* subLocality;
    /** The village, town or city. */
    @property (nonatomic, strong) NSString* locality;
    /** The island. */
    @property (nonatomic, strong) NSString* island;
    /** The subsidiary administrative area: district, parish, etc. */
    @property (nonatomic, strong) NSString* subAdminArea;
    /** The administrative area: state, province, etc. */
    @property (nonatomic, strong) NSString* adminArea;
    /** The country. */
    @property (nonatomic, strong) NSString* country;
    /** The postal code. */
    @property (nonatomic, strong) NSString* postCode;
    /**
    Gets the address as a string.
    If aFull is true supplies the administrative divisions
    (county, state, province, etc.) and country.
    */
    -(NSString*)ToString:(bool) aFull;
    @end

/** The CartoType path protocol. */
@protocol CartoTypePath
    /** Gets the number of contours (sub-paths) in the path. */
    -(int)contourCount;
    /** Gets the number of points in a certain contour. */
    -(int)pointCount:(int)aContourIndex;
    /** Gets a point, given its contour index and point index. Returns (0,0) if the contour index or point index is out of range. */
    -(CartoTypePathPoint)point:(int)aContourIndex pointIndex:(int)aPointIndex;
@end

/** Information returned by the match method of CartoTypeMapObject. */
@interface CartoTypeMatch: NSObject
    /** True if the text was found.*/
    @property (nonatomic) bool found;
    /** The name of the attribute in which the matched text was found. */
    @property (nonatomic, strong) NSString* key;
    /** The value of the attribute in which the matched text was found. */
    @property (nonatomic, strong) NSString* value;
    /** The start position of the matched text within the value. */
    @property (nonatomic) size_t start;
    /** The end position of the matched text within the value. */
    @property (nonatomic) size_t end;
@end

/** Constructs a 15-bit feature type code from 3 lower-case letters. */
#define MakeFeatureTypeCode(A,B,C) ((((A)-'a'+1) << 10) | (((B)-'a'+1) << 5) | ((C)-'a'+1))

/**
Every map object has a feature type.
The first 32 feature types are used for routable objects and their values are indexes into tables of speeds and weights in route profiles.
Other feature types are constructed from three-letter codes.

All feature type values must fit into 15 bits, and therefore must be in the range 0...32767.
*/
typedef enum
    {
    FeatureTypeMotorway = 0,
    FeatureTypeMotorwayLink = 1,
    FeatureTypeTrunkRoad = 2,
    FeatureTypeTrunkLink = 3,
    FeatureTypePrimaryRoad = 4,
    FeatureTypePrimaryLink = 5,
    FeatureTypeSecondaryRoad = 6,
    FeatureTypeSecondaryLink = 7,
    FeatureTypeTertiaryRoad = 8,
    FeatureTypeUnclassifiedRoad = 9,
    FeatureTypeResidentialRoad = 10,
    FeatureTypeTrack = 11,
    FeatureTypeServiceRoad = 12,
    FeatureTypePedestrianRoad = 13,
    FeatureTypeVehicularFerry = 14,
    FeatureTypePassengerFerry = 15,
    FeatureTypeLivingStreet = 16,
    FeatureTypeCycleway = 17,
    FeatureTypePath = 18,
    FeatureTypeFootway = 19,
    FeatureTypeBridleway = 20,
    FeatureTypeSteps = 21,
    FeatureTypeOtherRoad = 22,
    FeatureTypeUnpavedRoad = 23,
    FeatureTypeRailway = 24,
    FeatureTypeLightRail = 25,
    FeatureTypeSubway = 26,
    FeatureTypeAerialWay = 27,
    FeatureTypeSkiDownhill = 28,
    FeatureTypeSkiNordic = 29,
    FeatureTypeWaterway = 30,
    FeatureTypeUnknownRoute = 31,

    FeatureTypeUnknownNonRoute = 32,

    /*
    Three-letter type codes. Keep these in alphabetic order of type letters to avoid duplicates.
    No routable types should appear among the three-letter codes; all routable objects should be
    given one of the types 0...31.

    The lowest possible three-letter code, aaa, has the value 1057 = 1024 + 32 + 1.
    The highest code, zzz, has the value 27482 = (1024 * 26) + (32 * 26) + 26.
    */
    FeatureTypeAdminArea1 = MakeFeatureTypeCode('a','a','a'),
    FeatureTypeAdminArea2 = MakeFeatureTypeCode('a','a','b'),
    FeatureTypeAdminArea3 = MakeFeatureTypeCode('a','a','c'),
    FeatureTypeAdminArea4 = MakeFeatureTypeCode('a','a','d'),
    FeatureTypeAdminArea5 = MakeFeatureTypeCode('a','a','e'),
    FeatureTypeAdminArea6 = MakeFeatureTypeCode('a','a','f'),
    FeatureTypeAdminArea7 = MakeFeatureTypeCode('a','a','g'),
    FeatureTypeAdminArea8 = MakeFeatureTypeCode('a','a','h'),
    FeatureTypeAdminArea9 = MakeFeatureTypeCode('a','a','i'),
    FeatureTypeAdminArea10 = MakeFeatureTypeCode('a','a','j'),
    FeatureTypeAddress = MakeFeatureTypeCode('a','d','d'),
    FeatureTypeAerodrome = MakeFeatureTypeCode('a','e','r'),
    FeatureTypeAirport = MakeFeatureTypeCode('a','i','r'),
    FeatureTypeAirportGate = MakeFeatureTypeCode('a','g','t'),
    FeatureTypeAirportHoldingPosition = MakeFeatureTypeCode('a','h','p'),
    FeatureTypeAllotments = MakeFeatureTypeCode('a','l','l'),
    FeatureTypeAlpineHut = MakeFeatureTypeCode('a','l','p'),
    FeatureTypeAmbulanceStation = MakeFeatureTypeCode('a','m','b'),
    FeatureTypeApron = MakeFeatureTypeCode('a','p','r'),
    FeatureTypeAerialWayPylon = MakeFeatureTypeCode('a','p','y'),
    FeatureTypeArtwork = MakeFeatureTypeCode('a','r','t'),
    FeatureTypeAerialWayStation = MakeFeatureTypeCode('a','s','t'),
    FeatureTypeAirTerminal = MakeFeatureTypeCode('a','t','e'),
    FeatureTypeAtm = MakeFeatureTypeCode('a','t','m'),
    FeatureTypeAttraction = MakeFeatureTypeCode('a','t','t'),
    FeatureTypeBar = MakeFeatureTypeCode('b','a','r'),
    FeatureTypeBasin = MakeFeatureTypeCode('b','a','s'),
    FeatureTypeBay = MakeFeatureTypeCode('b','a','y'),
    FeatureTypeBeach = MakeFeatureTypeCode('b','e','a'),
    FeatureTypeBeacon = MakeFeatureTypeCode('b','e','c'),
    FeatureTypeBedAndBreakfast = MakeFeatureTypeCode('b','e','d'),
    FeatureTypeBench = MakeFeatureTypeCode('b','e','n'),
    FeatureTypeBank = MakeFeatureTypeCode('b','n','k'),
    FeatureTypeBoatyard = MakeFeatureTypeCode('b','o','a'),
    FeatureTypeBorough = MakeFeatureTypeCode('b','o','r'),
    FeatureTypeBoundary = MakeFeatureTypeCode('b','o','u'),
    FeatureTypeBicycleParking = MakeFeatureTypeCode('b','p','k'),
    FeatureTypeBicycleRental = MakeFeatureTypeCode('b','r','e'),
    FeatureTypeBareRock = MakeFeatureTypeCode('b','r','c'),
    FeatureTypeBarracks = MakeFeatureTypeCode('b','r','k'),
    FeatureTypeBrownField = MakeFeatureTypeCode('b','r','o'),
    FeatureTypeBusStop = MakeFeatureTypeCode('b','s','p'),
    FeatureTypeBusStation = MakeFeatureTypeCode('b','s','t'),
    FeatureTypeBufferStop = MakeFeatureTypeCode('b','u','f'),
    FeatureTypeBuilding = MakeFeatureTypeCode('b','u','i'),
    FeatureTypeBunker = MakeFeatureTypeCode('b','u','n'),
    FeatureTypeCabin = MakeFeatureTypeCode('c','a','b'),
    FeatureTypeCafe = MakeFeatureTypeCode('c','a','f'),
    FeatureTypeCampSite = MakeFeatureTypeCode('c','a','m'),
    FeatureTypeCanal = MakeFeatureTypeCode('c','a','n'),
    FeatureTypeCaveEntrance = MakeFeatureTypeCode('c','a','v'),
    FeatureTypeCableCar = MakeFeatureTypeCode('c','c','r'),
    FeatureTypeCableDistributionCabinet = MakeFeatureTypeCode('c','d','c'),
    FeatureTypeCemetery = MakeFeatureTypeCode('c','e','m'),
    FeatureTypeChairLift = MakeFeatureTypeCode('c','h','a'),
    FeatureTypeCheckPoint = MakeFeatureTypeCode('c','h','e'),
    FeatureTypeChalet = MakeFeatureTypeCode('c','h','l'),
    FeatureTypeCivilBoundary = MakeFeatureTypeCode('c','i','b'),
    FeatureTypeCinema = MakeFeatureTypeCode('c','i','n'),
    FeatureTypeCity = MakeFeatureTypeCode('c','i','t'),
    FeatureTypeCliff = MakeFeatureTypeCode('c','l','f'),
    FeatureTypeClinic = MakeFeatureTypeCode('c','l','i'),
    FeatureTypeCommercial = MakeFeatureTypeCode('c','m','r'),
    FeatureTypeCoastline = MakeFeatureTypeCode('c','o','a'),
    FeatureTypeCollege = MakeFeatureTypeCode('c','o','l'),
    FeatureTypeCommon = MakeFeatureTypeCode('c','o','m'),
    FeatureTypeConstruction = MakeFeatureTypeCode('c','n','s'),
    FeatureTypeConservation = MakeFeatureTypeCode('c','n','v'),
    FeatureTypeContinent = MakeFeatureTypeCode('c','o','n'),
    FeatureTypeCounty = MakeFeatureTypeCode('c','o','u'),
    FeatureTypeCarPark = MakeFeatureTypeCode('c','p','k'),
    FeatureTypeCarRental = MakeFeatureTypeCode('c','r','e'),
    FeatureTypeCrossing = MakeFeatureTypeCode('c','r','o'),
    FeatureTypeCountry = MakeFeatureTypeCode('c','r','y'),
    FeatureTypeCarSharing = MakeFeatureTypeCode('c','s','h'),
    FeatureTypeCutLine = MakeFeatureTypeCode('c','u','t'),
    FeatureTypeCarWash = MakeFeatureTypeCode('c','w','a'),
    FeatureTypeCaravanSite = MakeFeatureTypeCode('c','v','n'),
    FeatureTypeCyclingRoute = MakeFeatureTypeCode('c','y','r'),
    FeatureTypeDam = MakeFeatureTypeCode('d','a','m'),
    FeatureTypeDangerArea = MakeFeatureTypeCode('d','a','n'),
    FeatureTypeDentist = MakeFeatureTypeCode('d','e','n'),
    FeatureTypeDisused = MakeFeatureTypeCode('d','i','s'),
    FeatureTypeDitch = MakeFeatureTypeCode('d','i','t'),
    FeatureTypeDock = MakeFeatureTypeCode('d','o','c'),
    FeatureTypeDogPark = MakeFeatureTypeCode('d','o','g'),
    FeatureTypeDrain = MakeFeatureTypeCode('d','r','a'),
    FeatureTypeDragLift = MakeFeatureTypeCode('d','r','g'),
    FeatureTypeDoctors = MakeFeatureTypeCode('d','r','s'),
    FeatureTypeDistrict = MakeFeatureTypeCode('d','s','t'),
    FeatureTypeElectricVehicleCharging = MakeFeatureTypeCode('e','v','c'),
    FeatureTypeFarm = MakeFeatureTypeCode('f','a','r'),
    FeatureTypeFarmYard = MakeFeatureTypeCode('f','a','y'),
    FeatureTypeFell = MakeFeatureTypeCode('f','e','l'),
    FeatureTypeFerryTerminal = MakeFeatureTypeCode('f','e','t'),
    FeatureTypeFastFood = MakeFeatureTypeCode('f','f','d'),
    FeatureTypeFireExtinguisher = MakeFeatureTypeCode('f','i','e'),
    FeatureTypeFireFlapper = MakeFeatureTypeCode('f','i','f'),
    FeatureTypeFireHose = MakeFeatureTypeCode('f','i','h'),
    FeatureTypeFireStation = MakeFeatureTypeCode('f','i','s'),
    FeatureTypeFitnessStation = MakeFeatureTypeCode('f','i','t'),
    FeatureTypeFireHydrant = MakeFeatureTypeCode('f','i','y'),
    FeatureTypeForestry = MakeFeatureTypeCode('f','o','r'),
    FeatureTypeFishing = MakeFeatureTypeCode('f','s','h'),
    FeatureTypeFuel = MakeFeatureTypeCode('f','u','e'),
    FeatureTypeFunicular = MakeFeatureTypeCode('f','u','n'),
    FeatureTypeGarages = MakeFeatureTypeCode('g','a','r'),
    FeatureTypeGate = MakeFeatureTypeCode('g','a','t'),
    FeatureTypeGarden = MakeFeatureTypeCode('g','d','n'),
    FeatureTypeGenerator = MakeFeatureTypeCode('g','e','n'),
    FeatureTypeGreenHouse = MakeFeatureTypeCode('g','h','o'),
    FeatureTypeGlacier = MakeFeatureTypeCode('g','l','a'),
    FeatureTypeGolfCourse = MakeFeatureTypeCode('g','o','l'),
    FeatureTypeGondola = MakeFeatureTypeCode('g','o','n'),
    FeatureTypeGoodsAerialWay = MakeFeatureTypeCode('g','o','o'),
    FeatureTypeGrass = MakeFeatureTypeCode('g','r','a'),
    FeatureTypeGreenField = MakeFeatureTypeCode('g','r','e'),
    FeatureTypeGritBin = MakeFeatureTypeCode('g','r','b'),
    FeatureTypeGraveYard = MakeFeatureTypeCode('g','r','y'),
    FeatureTypeGuestHouse = MakeFeatureTypeCode('g','u','e'),
    FeatureTypeHalt = MakeFeatureTypeCode('h','a','l'),
    FeatureTypeHamlet = MakeFeatureTypeCode('h','a','m'),
    FeatureTypeHangar = MakeFeatureTypeCode('h','a','n'),
    FeatureTypeHeath = MakeFeatureTypeCode('h','e','a'),
    FeatureTypeHelipad = MakeFeatureTypeCode('h','e','l'),
    FeatureTypeRailwayHalt = MakeFeatureTypeCode('h','l','t'),
    FeatureTypeHikingRoute = MakeFeatureTypeCode('h','i','k'),
    FeatureTypeHorseRiding = MakeFeatureTypeCode('h','o','r'),
    FeatureTypeHospital = MakeFeatureTypeCode('h','o','s'),
    FeatureTypeHotel = MakeFeatureTypeCode('h','o','t'),
    FeatureTypeHorseRidingRoute = MakeFeatureTypeCode('h','r','r'),
    FeatureTypeHostel = MakeFeatureTypeCode('h','s','t'),
    FeatureTypeIceRink = MakeFeatureTypeCode('i','c','e'),
    FeatureTypeIndustrial = MakeFeatureTypeCode('i','n','d'),
    FeatureTypeInformation = MakeFeatureTypeCode('i','n','f'),
    FeatureTypeIsland = MakeFeatureTypeCode('i','s','l'),
    FeatureTypeIsolatedDwelling = MakeFeatureTypeCode('i','s','o'),
    FeatureTypeJunction = MakeFeatureTypeCode('j','c','t'),
    FeatureTypeKindergarten = MakeFeatureTypeCode('k','i','n'),
    FeatureTypeLandFill = MakeFeatureTypeCode('l','a','n'),
    FeatureTypeLand = MakeFeatureTypeCode('l','n','d'),
    FeatureTypeLevelCrossing = MakeFeatureTypeCode('l','e','v'),
    FeatureTypeLibrary = MakeFeatureTypeCode('l','i','b'),
    FeatureTypeLocality = MakeFeatureTypeCode('l','o','c'),
    FeatureTypeLockGate = MakeFeatureTypeCode('l','o','k'),
    FeatureTypeMaritimeBoundary = MakeFeatureTypeCode('m','a','b'),
    FeatureTypeMall = MakeFeatureTypeCode('m','a','l'),
    FeatureTypeMarsh = MakeFeatureTypeCode('m','a','r'),
    FeatureTypeMeadow = MakeFeatureTypeCode('m','e','a'),
    FeatureTypeMilitary = MakeFeatureTypeCode('m','i','l'),
    FeatureTypeMarina = MakeFeatureTypeCode('m','n','a'),
    FeatureTypeMotel = MakeFeatureTypeCode('m','o','t'),
    FeatureTypeMinorPowerLine = MakeFeatureTypeCode('m','p','l'),
    FeatureTypeMiniatureGolf = MakeFeatureTypeCode('m','r','g'),
    FeatureTypeMiniatureRailway = MakeFeatureTypeCode('m','r','y'),
    FeatureTypeMud = MakeFeatureTypeCode('m','u','d'),
    FeatureTypeMunicipality = MakeFeatureTypeCode('m','u','n'),
    FeatureTypeMuseum = MakeFeatureTypeCode('m','u','s'),
    FeatureTypeNatureReserve = MakeFeatureTypeCode('n','a','t'),
    FeatureTypeNationalPark = MakeFeatureTypeCode('n','a','p'),
    FeatureTypeNavalBase = MakeFeatureTypeCode('n','a','v'),
    FeatureTypeNeighborhood = MakeFeatureTypeCode('n','e','i'),
    FeatureTypeNursingHome = MakeFeatureTypeCode('n','u','r'),
    FeatureTypeOrchard = MakeFeatureTypeCode('o','r','c'),
    FeatureTypePrecisionApproachPathIndicator = MakeFeatureTypeCode('p','a','p'),
    FeatureTypePark = MakeFeatureTypeCode('p','a','r'),
    FeatureTypePublicBuilding = MakeFeatureTypeCode('p','b','u'),
    FeatureTypePostBox = MakeFeatureTypeCode('p','b','x'),
    FeatureTypePostCode = MakeFeatureTypeCode('p','c','o'),
    FeatureTypePicnicTable = MakeFeatureTypeCode('p','c','t'),
    FeatureTypePeak = MakeFeatureTypeCode('p','e','a'),
    FeatureTypePharmacy = MakeFeatureTypeCode('p','h','a'),
    FeatureTypePhone = MakeFeatureTypeCode('p','h','o'),
    FeatureTypePicnicSite = MakeFeatureTypeCode('p','i','c'),
    FeatureTypePier = MakeFeatureTypeCode('p','i','e'),
    FeatureTypePipeline = MakeFeatureTypeCode('p','i','p'),
    FeatureTypeParkingEntrance = MakeFeatureTypeCode('p','k','e'),
    FeatureTypeParking = MakeFeatureTypeCode('p','k','g'),
    FeatureTypeParkingSpace = MakeFeatureTypeCode('p','k','s'),
    FeatureTypePlantNursery = MakeFeatureTypeCode('p','l','a'),
    FeatureTypePlatform = MakeFeatureTypeCode('p','l','f'),
    FeatureTypePlaceOfWorship = MakeFeatureTypeCode('p','l','w'),
    FeatureTypePlayground = MakeFeatureTypeCode('p','l','y'),
    FeatureTypePostOffice = MakeFeatureTypeCode('p','o','f'),
    FeatureTypePolice = MakeFeatureTypeCode('p','o','l'),
    FeatureTypePosition = MakeFeatureTypeCode('p','o','s'),
    FeatureTypeProtectedArea = MakeFeatureTypeCode('p','r','a'),
    FeatureTypePowerSubStation = MakeFeatureTypeCode('p','s','s'),
    FeatureTypePub = MakeFeatureTypeCode('p','u','b'),
    FeatureTypePowerLine = MakeFeatureTypeCode('p','w','l'),
    FeatureTypePowerStation = MakeFeatureTypeCode('p','w','s'),
    FeatureTypeQuarry = MakeFeatureTypeCode('q','a','r'),
    FeatureTypeQuarter = MakeFeatureTypeCode('q','r','t'),
    FeatureTypeRange = MakeFeatureTypeCode('r','a','n'),
    FeatureTypeRapids = MakeFeatureTypeCode('r','a','p'),
    FeatureTypeRecycling = MakeFeatureTypeCode('r','c','y'),
    FeatureTypeRecreationGround = MakeFeatureTypeCode('r','e','c'),
    FeatureTypeReef = MakeFeatureTypeCode('r','e','e'),
    FeatureTypeRegion = MakeFeatureTypeCode('r','e','g'),
    FeatureTypeRetail = MakeFeatureTypeCode('r','e','t'),
    FeatureTypeRidge = MakeFeatureTypeCode('r','i','j'),
    FeatureTypeRiver = MakeFeatureTypeCode('r','i','v'),
    FeatureTypeRock = MakeFeatureTypeCode('r','o','c'),
    FeatureTypeRoundHouse = MakeFeatureTypeCode('r','o','u'),
    FeatureTypeResidentialArea = MakeFeatureTypeCode('r','s','d'),
    FeatureTypeRestaurant = MakeFeatureTypeCode('r','s','t'),
    FeatureTypeReservoir = MakeFeatureTypeCode('r','s','v'),
    FeatureTypeRunway = MakeFeatureTypeCode('r','u','n'),
    FeatureTypeRoute = MakeFeatureTypeCode('r','u','t'),
    FeatureTypeRiverBank = MakeFeatureTypeCode('r','v','b'),
    FeatureTypeSaltPond = MakeFeatureTypeCode('s','a','l'),
    FeatureTypeSand = MakeFeatureTypeCode('s','a','n'),
    FeatureTypeSauna = MakeFeatureTypeCode('s','a','u'),
    FeatureTypeSchool = MakeFeatureTypeCode('s','c','h'),
    FeatureTypeScree = MakeFeatureTypeCode('s','c','r'),
    FeatureTypeScrub = MakeFeatureTypeCode('s','c','b'),
    FeatureTypeSea = MakeFeatureTypeCode('s','e','a'),
    FeatureTypeStateEmergencyServiceStation = MakeFeatureTypeCode('s','e','s'),
    FeatureTypeShop = MakeFeatureTypeCode('s','h','o'),
    FeatureTypeSkiRoute = MakeFeatureTypeCode('s','k','r'),
    FeatureTypeSlipway = MakeFeatureTypeCode('s','l','i'),
    FeatureTypeSportsCenter = MakeFeatureTypeCode('s','p','o'),
    FeatureTypeSportsPitch = MakeFeatureTypeCode('s','p','p'),
    FeatureTypeSpring = MakeFeatureTypeCode('s','p','r'),
    FeatureTypeSportsTrack = MakeFeatureTypeCode('s','p','t'),
    FeatureTypeState = MakeFeatureTypeCode('s','t','a'),
    FeatureTypeStadium = MakeFeatureTypeCode('s','t','m'),
    FeatureTypeRailwayStation = MakeFeatureTypeCode('s','t','n'),
    FeatureTypeStation = MakeFeatureTypeCode('s','t','n'), // deliberate duplicate
    FeatureTypeStone = MakeFeatureTypeCode('s','t','o'),
    FeatureTypeStopPosition = MakeFeatureTypeCode('s','t','p'),
    FeatureTypeStream = MakeFeatureTypeCode('s','t','r'),
    FeatureTypeStrait = MakeFeatureTypeCode('s','t','t'),
    FeatureTypeSuburb = MakeFeatureTypeCode('s','u','b'),
    FeatureTypeSupermarket = MakeFeatureTypeCode('s','u','p'),
    FeatureTypeSurveyPoint = MakeFeatureTypeCode('s','u','r'),
    FeatureTypeSubwayEntrance = MakeFeatureTypeCode('s','w','e'),
    FeatureTypeSwimmingPool = MakeFeatureTypeCode('s','w','i'),
    FeatureTypeTank = MakeFeatureTypeCode('t','a','n'),
    FeatureTypeTaxi = MakeFeatureTypeCode('t','a','x'),
    FeatureTypeTheatre = MakeFeatureTypeCode('t','h','e'),
    FeatureTypeThemePark = MakeFeatureTypeCode('t','h','p'),
    FeatureTypeToilet = MakeFeatureTypeCode('t','o','i'),
    FeatureTypeTown = MakeFeatureTypeCode('t','o','w'),
    FeatureTypeTurningCircle = MakeFeatureTypeCode('t','c','i'),
    FeatureTypeTurningPoint = MakeFeatureTypeCode('t','p','t'),
    FeatureTypeTram = MakeFeatureTypeCode('t','r','a'),
    FeatureTypeTree = MakeFeatureTypeCode('t','r','e'),
    FeatureTypeTrafficSignals = MakeFeatureTypeCode('t','r','f'),
    FeatureTypeTrackPoint = MakeFeatureTypeCode('t','r','p'),
    FeatureTypeTreeRow = MakeFeatureTypeCode('t','r','r'),
    FeatureTypeTramStop = MakeFeatureTypeCode('t','r','s'),
    FeatureTypeTurnTable = MakeFeatureTypeCode('t','u','r'),
    FeatureTypeTower = MakeFeatureTypeCode('t','w','r'),
    FeatureTypeTaxiway = MakeFeatureTypeCode('t','w','y'),
    FeatureTypeUniversity = MakeFeatureTypeCode('u','n','i'),
    FeatureTypeVisualApproachSlopeIndicator = MakeFeatureTypeCode('v','a','s'),
    FeatureTypeVillageGreen = MakeFeatureTypeCode('v','i','g'),
    FeatureTypeVillage = MakeFeatureTypeCode('v','i','l'),
    FeatureTypeVineyard = MakeFeatureTypeCode('v','i','n'),
    FeatureTypeViewPoint = MakeFeatureTypeCode('v','i','w'),
    FeatureTypeVolcano = MakeFeatureTypeCode('v','o','l'),
    FeatureTypeWaterfall = MakeFeatureTypeCode('w','a','f'),
    FeatureTypeWaterPark = MakeFeatureTypeCode('w','a','p'),
    FeatureTypeWater = MakeFeatureTypeCode('w','a','t'),
    FeatureTypeWeir = MakeFeatureTypeCode('w','e','r'),
    FeatureTypeWetland = MakeFeatureTypeCode('w','e','t'),
    FeatureTypeWindsock = MakeFeatureTypeCode('w','i','s'),
    FeatureTypeWalkingRoute = MakeFeatureTypeCode('w','l','k'),
    FeatureTypeWood = MakeFeatureTypeCode('w','o','o'),
    FeatureTypeWorks = MakeFeatureTypeCode('w','o','r'),
    FeatureTypeWaypoint = MakeFeatureTypeCode('w','p','t'),
    FeatureTypeWaterTower = MakeFeatureTypeCode('w','t','t'),
    FeatureTypeWaterWell = MakeFeatureTypeCode('w','t','w'),
    FeatureTypeZoo = MakeFeatureTypeCode('z','o','o'),

    // legacy types for backward compatibility
    FeatureTypeLegacyBridleway = MakeFeatureTypeCode('b','r','i'),
    FeatureTypeLegacyCycleway = MakeFeatureTypeCode('c','y','c'),
    FeatureTypeLegacyFootway = MakeFeatureTypeCode('f','o','o'),
    FeatureTypeLegacyLightRail = MakeFeatureTypeCode('l','i','r'),
    FeatureTypeLegacyMonorail = MakeFeatureTypeCode('m','o','n'),
    FeatureTypeLegacyNarrowGauge = MakeFeatureTypeCode('n','a','r'),
    FeatureTypeLegacyPreservedRailway = MakeFeatureTypeCode('p','r','y'),
    FeatureTypeLegacyRailway = MakeFeatureTypeCode('r','l','y'),
    FeatureTypeLegacySteps = MakeFeatureTypeCode('s','t','e'),
    FeatureTypeLegacySubway = MakeFeatureTypeCode('s','w','y'),
    FeatureTypeLegacyTram = MakeFeatureTypeCode('t','r','a'),

    FeatureTypeInvalid = INT16_MAX
    } CartoTypeFeatureType;
    
/** The feature discriminator stored in a FeatureInfo object. */
typedef enum
    {
    /** Not a route. */
    FeatureDiscriminatorNonRoute = 0,
    /** A route that is a bridge. */
    FeatureDiscriminatorBridgeRoute = 1,
    /** A route that is a tunnel. */
    FeatureDiscriminatorTunnelRoute = 2,
    /** A route that is neither a tunnel nor a bridge. */
    FeatureDiscriminatorRoute = 3
    } CartoTypeFeatureDiscriminator;

/**
Feature information for a map object, represented internally as a 32-bit value.

Every map object has a feature info value. Feature info values fall into two categories: route and non-route.

Route values have a feature type in the range 0...31 (FeatureTypeMotorway...FeatureTypeUnknownRoute); it is used as the index into the speed and bonus
arrays in a route profile. Route values have other information relevant to routing including the speed limit
and access restrictions.

Non-route values have a feature type in the range 32...32767. Non-route feature types, apart from the special values
FeatureTypeUnknownNonRoute (32) and FeatureTypeInvalid (32767), may be expressed as three-letter mnemonics, which can be constructed using
the FeatureTypeCode function. Non-route values have an 11-bit sub-type that may be used for fine distinctions such
as settlement rank or boundary type.

Both route and non-route values have a level in the range -8...7, where 0 represents ground level. The level is used as
one of the criteria determining drawing order.
*/
@interface CartoTypeFeatureInfo: NSObject
    /** Creates a feature info object of type UnknownNonRoute. */
    -(id)init;
/** Creates a feature info object from a feature type. */
    -(id)initWithType:(CartoTypeFeatureType)aFeatureType;
    /**
    Creates a feature info object with a given type and sub-type.
    The sub-type is clamped to the range 0...2047, and is ignored if aType is a route type,
    because routes do not have sub-types.
    */
    -(id)initWithTypeAndSubType:(CartoTypeFeatureType)aFeatureType subType:(int)aSubType;
    /**
    Creates a non-route feature info object of type UnknownNonRoute and a given sub-type.
    The sub-type is clamped to the range 0...2047.
    */
    -(id)initWithSubType:(int)aSubType;
    /** The feature discriminator. */
    @property (nonatomic) CartoTypeFeatureDiscriminator discriminator;
    /** True if this is a route. */
    @property (nonatomic) bool route;
    /** True if this is a route that is a tunnel. Only route objects can made into tunnels. */
    @property (nonatomic) bool tunnel;
    /** True if this is a route that is a bridge. Only route objects can made into bridges. */
    @property (nonatomic) bool bridge;
    /** The level, which is in the range -8 ... 7. */
    @property (nonatomic) int level;
    /** The feature type. Route objects can only be set to route types, and non-route objects to non-route types. */
    @property (nonatomic) CartoTypeFeatureType type;
    /** True if this is a one-way route. Non-route objects cannot be set as one-way.  */
    @property (nonatomic) bool oneWay;
    /** True if this is a one-way route in the direction in which the route is stored. */
    @property (nonatomic) bool oneWayForward;
    /** True if this is a one-way route in the direction opposite to the one in which the route is stored. */
    @property (nonatomic) bool oneWayBackward;
    /** True if this is a two-way route and the rule of the road is to drive on the left. The rule of the road cannot be set for non-route objects. */
    @property (nonatomic) bool driveOnLeft;
    /** Returns true if this is a two-way route and the rule of the road is to drive on the right. The rule of the road cannot be set for non-route objects. */
    @property (nonatomic) bool driveOnRight;
    /** Reverses the one-way direction if this is a one-way route. */
    -(void)reverseOneWayDirection;
    /** True if this is a route that is part of a roundabout. This property cannot be set for non-route objects. */
    @property (nonatomic) bool roundabout;
    /** True if this is a toll route. This property cannot be set for non-route objects.  */
    @property (nonatomic) bool toll;
    /** True if this is a route with a one-way direction opposite to the order of its points. This property cannot be set for non-route objects. */
    @property (nonatomic) bool wrongWay;
    /** The speed limit in kph. A value of zero indicates that no speed limit is known. This property cannot be set for non-route objects. */
    @property (nonatomic) int speedLimit;
    /** The gradient as a number in the range 0...7. Values 0...3 are uphill and 4...7 are downhill. Gradients are stored only in route arcs and not usually in other map objects. This property cannot be set for non-route objects. */
    @property (nonatomic) int gradient;
    /** Sets or clears the vehicle access flags. Throws an exception if this is not a route. */
    -(void)setVehicleAccess:(bool)aValue;
    /** True if this is a route and pedestrian access is allowed. This property cannot be set for non-route objects. */
    @property (nonatomic) bool pedestrianAccess;
    /** True if this is a route and cycle access is allowed. */
    @property (nonatomic) bool cycleAccess;
    /** True if this is a route and motor vehicle access is allowed. This property cannot be set for non-route objects. */
    @property (nonatomic) bool motorVehicleAccess;
    /** True if this is a route and emergency vehicle access is allowed. */
    @property (nonatomic) bool emergencyVehicleAccess;
    /** True if this is a route and there are access restrictions other than those specified by the standard flags. This property cannot be set for non-route objects. */
    @property (nonatomic) bool otherAccessRestricted;
    /** True if this is a route and no normal access is allowed. */
    @property (nonatomic) bool isPrivate;
    /** The sub-type. Always 0 if this is a route. This property cannot be set for route objects. */
    @property (nonatomic) int subType;
    /** Creates a feature info object from a raw value. For internal use only. */
    -(id)initWithRawValue:(uint32_t)aValue;
    /** Returns the raw value. For internal use only. */
    -(uint32_t)rawValue;
@end

/** The number of feature types that are used for routes. These values are also indexes into the speed and bonus arrays in route profiles. */
static const uint32_t KFeatureInfoRouteTypeCount = 32;
/** The number of different gradients distinguished by a feature info or route profile object. */
static const uint32_t KFeatureInfoGradientCount = 8;
/** The maximum value for the sub-type of a non-route feature info object. The minimum value is zero. */
static const uint32_t KFeatureInfoMaxSubType = 2047;

/** The amount by which the route access flags are shifted. */
static const uint32_t KRouteAccessShift = 26;
/** A mask to select the route access flags. */
static const uint32_t KRouteAccessMask = 63U << KRouteAccessShift;
/** A flag indicating that this route is one-way but is stored in the opposite direction to the allowed direction of travel. */
static const uint32_t KRouteAccessWrongWayFlag = 1 << 26;
/** A flag to forbid pedestrian access in a feature info object, or, in a vehicle type, to indicate a pedestrian. */
static const uint32_t KRouteAccessPedestrianFlag = 1 << 27;
/** A flag to forbid cycle access in a feature info object, or, in a vehicle type, to indicate a cyclist. */
static const uint32_t KRouteAccessCycleFlag = 1 << 28;
/** A flag to forbid motor vehicle access in a feature info object, or, in a vehicle type, to indicate a motor vehicle. */
static const uint32_t KRouteAccessMotorVehicleFlag = 1 << 29;
/** A flag to forbid emergency vehicle access in a feature info object, or, in a vehicle type, to indicate an emergency vehicle. */
static const uint32_t KRouteAccessEmergencyVehicleFlag = 1 << 30;
/** A flag to indicate the presence of other access restrictions in a feature info object, or, in a vehicle type, to indicate a type of vehicle not covered by the other flags. */
static const uint32_t KRouteAccessOtherFlag = 1U << 31;
/** A set of flags covering all vehicles. */
static const uint32_t KRouteAccessVehicle = KRouteAccessCycleFlag | KRouteAccessMotorVehicleFlag | KRouteAccessEmergencyVehicleFlag;
/** A set of flags indicating normal access: that is, access for pedestrians, cyclists and motor vehicles. */
static const uint32_t KRouteAccessNormal = KRouteAccessCycleFlag | KRouteAccessMotorVehicleFlag | KRouteAccessPedestrianFlag;

/** A map object: a point, line, polygon or array (texture), with its name, layer, ID and other attributes. */
@interface CartoTypeMapObject: NSObject <CartoTypePath>
    /** Initializes a CartoTypeMapObject with a C++ map object; for internal use only. */
    -(id)initWithMapObject:(void*)aMapObject;
    /** Destroys the map object. */
    -(void)dealloc;
    /** The type of the map object: point, line, polygon or array. */
    @property (nonatomic) CartoTypeMapObjectType type;
    /** The name of the map layer to which the map object belongs. */
    @property (nonatomic) NSString* layerName;
    /** The label or standard name of the map object. */
    @property (nonatomic) NSString* label;
    /** Gets any string attribute. If aAttribName is null, gets the label. */
    -(NSString*)stringAttribute:(NSString*)aAttribName;
    /** Gets all the string attributes as a dictionary of key-value pairs. */
    -(NSMutableDictionary*)stringAttributes;
    /** The feature info of a map object. */
    @property (nonatomic) CartoTypeFeatureInfo* featureInfo;
    /**
    The center of a map object in map coordinates.
    The center is defined as the center of gravity for polygons and arrays,
    a point half way along the path for lines, and the point itself for points.
    */
    @property (nonatomic) CartoTypePoint center;
    
    /** The axis-aligned bounding box of a map object, in map coordinates. */
    @property (nonatomic) CartoTypeRect bounds;

    /**
    The center of a map object in degrees.
    The center is defined as the center of gravity for polygons and arrays,
    a point half way along the path for lines, and the point itself for points.
    Because the center is calculated using map coordinates then converted to degrees,
    it depends on the map projection used by the map object.
    */
    @property (nonatomic) CartoTypePointAndResult centerInDegrees;

    /** The bounding box of a map object in degrees. */
    @property (nonatomic) CartoTypeRectAndResult boundsInDegrees;
    
    /** The geometry of a map object in degrees. */
    @property (nonatomic) CartoTypeGeometryAndResult* geometryInDegrees;
    /**
    Finds the first string attribute matching aText, using aMatchMethod, and returns
    information about it.
    If aAttributes is non-null this function examines only attributes in the supplied comma-separated list,
    otherwise it examines all attributes except those starting with an underscore, which by convention are non-textual.
    Attribute names may contain the wild cards ? and *.
    If aPhrase is true matches of phrases within the attribute are allowed.
    */
    -(CartoTypeMatch*)match:(NSString*)aText matchMethod:(CartoTypeStringMatchMethod)aMethod attributes:(NSString*)aAttributes phrase:(bool)aPhrase;
    /** The object ID of the map object, which can be used to delete objects inserted at run-time. */
    @property (nonatomic) uint64_t objectId;
    /** Gets the number of contours (sub-paths) in the path. */
    -(int)contourCount;
    /** Gets the number of points in a certain contour. */
    -(int)pointCount:(int)aContourIndex;
    /** Gets a point, given its contour index and point index. Returns (0,0) if the contour index or point index is out of range. */
    -(CartoTypePathPoint)point:(int)aContourIndex pointIndex:(int)aPointIndex;
    /** The area of the map object in square meters. Returns zero for a point object. */
    @property (nonatomic) double area;
    /** The length or perimeter of the map object in meters. Returns zero for a point object. */
    @property (nonatomic) double lengthOrPerimeter;

    /** Gets a pointer to the C++ map object; for internal use only. */
    -(void*)object;
@end

/** A group of map objects with the same name, returned by a search function. */
@interface CartoTypeMapObjectGroup: NSObject
    /** Initializes a CartoTypeMapObjectGroup with a C++ CMapObjectGroup; for internal use only. */
    -(id)initWithMapObjectGroup:(void*)aMapObjectGroup;
    /** The name, summary address or other common attribute of all the objects. */
    @property (nonatomic, strong) NSString* name;
    /** The map objects. */
    @property (nonatomic, strong) NSMutableArray* objectArray;
@end

/** The ways two paths can intersect. */
typedef enum
    {
    /** The intersection type is unknown. */
    PathIntersectionTypeUnknown,
    /** The paths do not intersect. */
    PathIntersectionTypeSeparate,
    /** The paths intersect or are identical. */
    PathIntersectionTypeIntersects,
    /** The first path contains the second. */
    PathIntersectionTypeContains,
    /** The second path contains the first. */
    PathIntersectionTypeContained
    } CartoTypePathIntersectionType;

/** Information about the intersection of two paths and their distance apart. */
@interface CartoTypePathIntersectionInfo: NSObject
    /** The intersection type. */
    @property (nonatomic) CartoTypePathIntersectionType type;
    /**
    The distance between the paths. The units are meters for functions involving
    map objects, otherwise they are the units used by the paths.
    */
    @property (nonatomic) double distance;
    /**
    The nearest point on the first path. The units are latitude and longitude for functions involving
    map objects, otherwise they are the units used by the paths.
    */
    @property (nonatomic) CartoTypePoint nearest1;
    /**
    The nearest point on the second path. The units are latitude and longitude for functions involving
    map objects, otherwise they are the units used by the paths.
    */
    @property (nonatomic) CartoTypePoint nearest2;

    /** For internal use: gets a pointer to the TPathIntersectionInfo object. */
    -(void*)pointer;
@end

/** Types of clipping done by the Clip functions of the Geometry class. */
typedef enum
    {
    /** Returns the intersection of two paths; commutative. */
    IntersectionClipOperation,
    /** Returns the union of two paths; commutative. */
    UnionClipOperation,
    /** Returns the difference of two paths; non-commutative. */
    DifferenceClipOperation,
    /** Returns the exclusive-or of the two paths; that is, any regions which are in neither path; commutative. */
    XorClipOperation
    }  CartoTypeClipOperation;

/** Geometry: a point, polyline or poly-polygon with a coordinate type. */
@interface CartoTypeGeometry: NSObject
    /** Initializes the geometry object with the coordinate type to be used when adding points. */
    -(id)initWithCoordType:(CartoTypeCoordType)aCoordType;
    /** Creates a geometry object from a map object. */
    -(id)initWithMapObject:(CartoTypeMapObject*)aMapObject;
    /** Creates a geometry object from a single contour of another geometry object. */
    -(id)initWithGeometryContour:(CartoTypeGeometry*)aGeometry contourIndex:(int)aContourIndex;
    /** Creates a geometry object from a rectangle. */
    -(id)initWithRect:(CartoTypeRect)aRect coordType:(CartoTypeCoordType)aCoordType;
    /** Destroys the geometry object. */
    -(void)dealloc;
    /** Returns the coordinate type. */
    -(CartoTypeCoordType)coordType;
    /** Adds a new point. */
    -(void)appendX:(double)aX andY:(double)aY;
    /** Adds a new point, specifying the point type as well as the coordinates. */
    -(void)appendX:(double)aX andY:(double)aY andPointType:(CartoTypePointType)aPointType;
    /** Starts a new contour. */
    -(void)beginContour;
    /** Returns the number of contours. */
    -(int)contourCount;
    /** Returns true if a contour is an outer contour, defined as being anti-clockwise, that is, having a positive area when treating all points as on-curve. Useful only for closed geometries. */
    -(bool)contourIsOuter:(int)aContourIndex;
    /** Returns the number of points in a contour. */
    -(int)pointCount:(int)aContourIndex;
    /** Returns a point, given a contour index and a point index. */
    -(CartoTypePathPointFP)point:(int)aContourIndex pointIndex:(int)aPointIndex;
    /** Returns true if the geometry has no points. */
    -(bool)isEmpty;
    /** Returns true if the geometry is made from closed paths. */
    -(bool)isClosed;
    /** Sets the closed state. */
    -(void)setClosed:(bool)aClosed;
    /** Reverses the order of the points in each contour. */
    -(void)reverse;
    /** Interpolates points at a maximum distance apart to convert lines to great circles. Works only with degree coordinates, and with on-curve points. */
    -(CTResult)convertLinesToGreatCircles:(double)aMaxDistanceInMeters;
    /** Returns the intersection type of this geometry with a map object. */
    -(CartoTypePathIntersectionType)intersectionType:(CartoTypeMapObject*)aMapObject;
    /**
    Returns information about the intersection of this geometry with a map object.
    The nearest positions are in degrees (X = longitude, Y = latitude) and the the distance is in meters.
    */
    -(CartoTypePathIntersectionInfo*)intersectionInfo:(CartoTypeMapObject*)aMapObject;
    /** Returns the intersection types of this geometry with an array of map objects.  The types are returned as integers because of the difficulty of returning an array of enumeration values in Objective C. */
    -(NSArray<NSNumber*>*)intersectionTypes:(NSArray<CartoTypeMapObjectGroup*>*)aMapObjectArray;
    /**
    Returns information about the intersection of this geometry with a sequence of map objects returned by a function.
    The nearest positions are in degrees (X = longitude, Y = latitude) and the the distances are in meters.
    */
    -(NSArray<CartoTypePathIntersectionInfo*>*)intersectionInfos:(NSArray<CartoTypeMapObjectGroup*>*)aMapObjectArray;
    /** Returns true if this geometry intersects, contains or is contained by the map object aMapObject. */
    -(bool) intersects:(CartoTypeMapObject*)aMapObject;
    /** Returns true if this geometry contains the map object aMapObject. */
    -(bool) contains:(CartoTypeMapObject*)aMapObject;
    /** Returns true if this geometry is contained by the map object aMapObject. */
    -(bool) contained:(CartoTypeMapObject*)aMapObject;
    /**
    Returns the result of clipping a map object using this geometry, which must be closed.
    Supports intersection, union, difference and exclusive-or operations.
    All operations except difference are commutative.
    The difference operation subtracts this geometry from aMapObject.
    
    The returned geometry object is in degrees.
    */
    -(CartoTypeGeometry*) clip:(CartoTypeClipOperation)aClipOperation mapObject:(CartoTypeMapObject*)aMapObject;
    /**
    Returns the results of clipping an array of map objects using this geometry, which must be closed.
    Supports intersection, union, difference and exclusive-or operations.
    All operations except difference are commutative.
    The difference operation subtracts this geometry from the map objects.
    
    The returned geometry objects are in degrees.
    */
    -(NSArray<CartoTypeGeometry*>*) clip:(CartoTypeClipOperation)aClipOperation mapObjects:(NSArray<CartoTypeMapObjectGroup*>*)aMapObjectArray;
    /**
    Returns a geometry that is the envelope of a map object,
    created by offsetting the path by aOffset. The offset
    may be negative if the subject path is made of polygons only.
    
    The returned geometry object is in degrees.
    */
    +(CartoTypeGeometry*) envelope:(CartoTypeMapObject*)aMapObject offsetInMeters:(double)aOffsetInMeters;

    /** (for internal use) Initializes the geometry object with an underlying C++ geometry object. */
    -(id)initWithGeometry:(void*)aGeometry;
    /** (for internal use) Gets a pointer to the underlying C++ geometry object. */
    -(void*)geometry;
@end

/** Parameters for the general find function. */
@interface CartoTypeFindParam: NSObject
    /** The default initializer. */
    -(id)init;
    /** Creates find parameters for finding nearby. */
    -(id)initWithFeatureType:(CartoTypeFeatureType)aFeatureType text:(NSString*)aText location:(CartoTypeGeometry*)aLocation;

    /** The maximum number of objects to return. */
    @property (nonatomic) int maxObjectCount;
    /** The clip path; no clipping is done if clip is empty. */
    @property (nonatomic) CartoTypeGeometry* clip;
    /** The current location. If it is non-empty, objects in or near this region are preferred. */
    @property (nonatomic) CartoTypeGeometry* location;
    /** If non-null, a list separated by spaces or commas. If it is null all layers are searched. */
    @property (nonatomic, strong) NSString* layers;
    /**
    If 'attributes' is null or empty, search all the attributes, otherwise it is a list of
    attributes separated by spaces or commas; use '$' to indicate the label (the unnamed attribute).
    */
    @property (nonatomic, strong) NSString* attributes;
    /**
    If non-null, restricts the search to objects containing a string
    in one of their string attributes.
    */
    @property (nonatomic, strong) NSString* text;
    /** The string matching method used for text searching. */
    @property (nonatomic) CartoTypeStringMatchMethod stringMatchMethod;
    /**
    If non-null, a condition expression (e.g., "Type==2")
    which must be fulfilled by all the objects.
    */
    @property (nonatomic, strong) NSString* condition;
    /** If merge is true adjoining objects with the same name and attributes may be merged into a single object. The default value is false. */
    @property (nonatomic) bool merge;
    /**
    The maximum time in seconds allowed for a find operation. Find operations are not guaranteed to
    return in this time or less, but will attempt to do so. A timeout of zero or less means there is no timeout.
    */
    @property (nonatomic) double timeOut;
@end

/** Types of writable maps. */
typedef enum
    {
    /** A writable map stored in memory. */
    MemoryWritableMapType,
    /** A writable map stored in an SQLite database. */
    SQLiteWritableMapType
    } CartoTypeWritableMapType;

/** Positions for notices like the legend or scale bar. */
typedef enum
    {
    /** The top left corner of the display. */
    NoticePositionTopLeft,
    /** The top right corner of the display. */
    NoticePositionTopRight,
    /** The bottom left corner of the display. */
    NoticePositionBottomLeft,
    /** The bottom right corner of the display. */
    NoticePositionBottomRight,
    /** The center of the top edge of the display. */
    NoticePositionTop,
    /** The center of the right side of the display. */
    NoticePositionRight,
    /** The center of the left side of the display. */
    NoticePositionLeft,
    /** The center of the bottom edge of the display. */
    NoticePositionBottom,
    /** The center of the display. */
    NoticePositionCenter
    } CartoTypeNoticePosition;

/**
Positions for notices like the legend or scale bar.
 
Notices are positioned by moving them inwards from the designated position
by the specified insets. For central positions (e.g., the X inset in Top,
or both insets in Center) X insets move the position right and Y insets move it down.
Units may be 'cm', 'mm', 'in', 'pt' (points: 1/72in), or 'pc' (picas: 1/6in);
anything else, including null, means pixels.
 
The default inset is 3mm, or zero for central positions.
*/
@interface CartoTypeExtendedNoticePosition: NSObject
    /** Creates an extended notice position from a base position, X and Y insets, and units in which the insets are defined. */
    -(id)init:(CartoTypeNoticePosition)aPosition xInset:(double)aXInset xUnit:(NSString*)aXUnit yInset:(double)aYInset yUnit:(NSString*)aYUnit;
    /** Returns the native position object. For internal use only. */
    -(void*)position;
@end

/** A bitmap and a position to draw it. Used when drawing notices on the map. */
@interface CartoTypePositionedBitmap: NSObject
    /** Creates a positioned bitmap from a bitmap and a position. */
    -(id)init:(CGImageRef)aImage topLeft:(CGPoint)aTopLeft;
    /** Destroys the object. */
    -(void)dealloc;
    /** The bitmap. */
    @property (nonatomic) CGImageRef image;
    /** The position at which to draw the bitmap on the map. */
    @property (nonatomic) CGPoint topLeft;
@end

/** Parameters giving detailed control of the perspective view. */
@interface CartoTypePerspectiveParam: NSObject
    /** Reads a perspective parameter object from its XML form. */
    -(CTResult)readFromXml:(NSString*)aText;
    /** Creates a string representing a perspective parameter object in XML. */
    -(NSString*)toXml;
    /** The position of the point on the terrain below the camera, in degrees longitude (x) and latitude (y). */
    @property (nonatomic) CartoTypePoint positionDegrees;
    /** If true, ignore iPosition, and set the camera position so that the location in the center of the display is shown. */
    @property (nonatomic) bool autoPosition;
    /** The height of the camera above the terrain. The value 0 causes a default value to be used, which is the width of the display. */
    @property (nonatomic) double heightMeters;
    /** The azimuth of the camera in degrees going clockwise, where 0 is N, 90 is E, etc. */
    @property (nonatomic) double azimuthDegrees;
    /** If true, ignore iAzimuthDegrees and use the current map orientation. */
    @property (nonatomic) bool autoAzimuth;
    /** The declination of the camera downward from the horizontal plane. Values are clamped to the range -90...90. */
    @property (nonatomic) double declinationDegrees;
    /** The amount by which the camera is rotated about its axis, after applying the declination, in degrees going clockwise. */
    @property (nonatomic) double rotationDegrees;
    /** The camera's field of view in degrees. */
    @property (nonatomic) double fieldOfViewDegrees;
    /** (for internal use) Gets a pointer to the underlying C++ object. */
    -(void*)getPerspectiveParam;
@end

/** The view state, which can be retrieved or set so that maps can be recreated. */
@interface CartoTypeViewState: NSObject
    /** Reads a view state from its XML form. */
    -(CTResult)readFromXml:(NSString*)aText;
    /** Creates a string representing a view state in XML. */
    -(NSString*)toXml;
    /** The display width in pixels. */
    @property (nonatomic) int widthInPixels;
    /** The display height in pixels. */
    @property (nonatomic) int heightInPixels;
    /** The view center in map coordinates. */
    @property (nonatomic) CartoTypePoint viewCenterDegrees;
    /** The denominator of the scale fraction; e.g., 50000 for 1:50000. */
    @property (nonatomic) double scaleDenominator;
    /** The clockwise rotation of the view in degrees. */
    @property (nonatomic) double rotationDegrees;
    /** True if perspective mode is on. */
    @property (nonatomic) bool perspective;
    /** Gets a copy of the parameters to be used for perspective mode. */
    -(CartoTypePerspectiveParam*)getPerspectiveParam;
    /** Sets the parameters to be used for perspective mode. */
    -(void)setPerspectiveParam:(CartoTypePerspectiveParam*)aPerspectiveParam;
    /** (for internal use) Gets a pointer to the underlying C++ object. */
    -(void*)viewState;
@end

/** The case of letters and strings in Latin, Greek, Cyrillic and other alphabetic scripts. */
typedef enum
    {
    /** A code used when the letter case is to be left as it is, or is irrelevant. */
    LetterCaseNone,
    /** The case of the letters 'a', 'b', c', etc. */
    LetterCaseLower,
    /** The case of the letters 'A', 'B', 'C, etc. */
    LetterCaseUpper,
    /**
    The case of letters like U+01C8 'Lj' and a few others; also indicates
    that the first letter of each word should be changed to titlecase.
    */
    LetterCaseTitle
    } CartoTypeLetterCase;
    
/* Identifiers of tables in CTM1 map files. */

/** The ID of the global information table. */
static const uint16_t KGlobalTable = 0;
/** The ID of the table containing the layers containing the map objects; see also KLowResolutionLayerTable. */
static const uint16_t KLayerTable = 1;
/** The ID of the obsolete text index table. */
static const uint16_t KTable2Obsolete = 2;
/** The ID of the obsolete projection table used up to CTM1 version 3.0. */
static const uint16_t KTable3Obsolete = 3;
/** The ID of the the table containing the map projection. */
static const uint16_t KProjectionTable = 4;
/** The ID of the table of compressed strings referenced by the map objects. */
static const uint16_t KStringTable = 5;
/** The ID of the table containing layer data containing map objects at lower resolutions, simplified for display at smaller scales; see also KLayerTable. */
static const uint16_t KLowResolutionLayerTable = 6;
/** The ID of the table containing color palettes for raster image objects. */
static const uint16_t KPaletteTable = 7;
/** The ID of the table containing the text index used when searching for string attributes. */
static const uint16_t KTextIndexTable = 8;
/** The ID of the table containing the obsolete serialised A-star routing network used up to format version 4.1. */
static const uint16_t KTable9Obsolete = 9;
/** The ID of the A-star routing data table. */
static const uint16_t KRouteTableAStar = 10;
/** The ID of the contraction hierarchy routing data table. */
static const uint16_t KRouteTableCH = 11;
/** The ID of the table containing extra information used for A-star routing. */
static const uint16_t KRouteTableExtra = 12;
/** The ID of the turn-expanded routing data table. */
static const uint16_t KRouteTableTurnExpanded = 13;
/** The ID of the table containing contraction hierarchy routing data that can optionally be used stand-alone, in combination with other map files. */
static const uint16_t KRouteTableCHStandAlone = 14;
/** The ID of the turn-expanded contraction hierarchy routing data table. */
static const uint16_t KRouteTableTECH = 15;
/** The ID of the table containing contraction hierarchy routing data that is a tile that can be used with other tiles from the same data set. */
static const uint16_t KRouteTableCHTiled = 16;
/** The ID of the table containing turn-expanded contraction hierarchy routing data that is a tile that can be used with other tiles from the same data set. */
static const uint16_t KRouteTableTECHTiled = 17;
/** The ID of the table containing compact turn-expanded routing data, which uses less run-time RAM. */
static const uint16_t KRouteTableTurnExpandedCompact = 18;
/** An ID used when no table exists or the table type is unknown. */
static const uint16_t KNoTable = UINT16_MAX;

/**
The format used for points in CTM1 data.
These numbers must fit into 8 bits for the moment because of the way they are stored in the CTM1 file.
The values are discontinuous because several obsolete values have been discarded.
*/
typedef enum
    {
    /** An arbitrary or unknown point format. */
    PointFormatUnknown = 0,

    /** Units are projected map meters. */
    PointFormatMeter = 2,

    /**
    Units are degrees as 11.21 fixed-point numbers.
    That is, there are 2^21 (2097152) units to a degree.
    This format is used only by the makemap tool, for representing
    unprojected map data.
    */
    PointFormatScaledDegree = 7,

    /**
    Units are 32nds of projected map meters.
    This is the default format for CTM1 data.
    */
    PointFormatMeter32nds = 8
    } CartoTypePointFormat;

/** Metadata describing a CTM1 map file. */
@interface CartoTypeMapMetaData: NSObject
    /** The major part of the CTM1 format version. */
    @property (nonatomic) int fileVersionMajor;
    /** The minor part of the CTM1 format version. */
    @property (nonatomic) int fileVersionMinor;
    /** The major part of the version of CartoType used to build the makemap tool which created the CTM1 file. */
    @property (nonatomic) int cartoTypeVersionMajor;
    /** The minor part of the version of CartoType used to build the makemap tool which created the CTM1 file. */
    @property (nonatomic) int cartoTypeVersionMinor;
    /** The build (version control revision number) used to build the makemap tool which created the CTM1 file. */
    @property (nonatomic) int cartoTypeBuild;
    /** The name of the map data set. */
    @property (nonatomic) NSString* dataSetName;
    /** The copyright notice applying to the map data. */
    @property (nonatomic) NSString* copyright;
    /** The name of the map projection. */
    @property (nonatomic) NSString* projectionName;
    /** The layers. */
    @property (nonatomic) NSMutableArray<NSString*>* layers;
    /** The Proj4 parameters for the map projection. */
    @property (nonatomic) NSString* projectionParameters;
    /** The point format for map coordinates: either Meter or Meter32nds. */
    @property (nonatomic) CartoTypePointFormat pointFormat;
    /** The axis-aligned bounds of the map, in map coordinates. */
    @property (nonatomic) CartoTypeRect extentInMapCoords;
    /** The axis-aligned bounds of the map, in degrees of longitude and latitude. */
    @property (nonatomic) CartoTypeRect extentInDegrees;
    /** The route table type: one of the constants KRouteTableAStar, KRouteTableCH, etc. */
    @property (nonatomic) int routeTableType;
    /** True if the route data contains gradients. */
    @property (nonatomic) bool routeDataHasGradients;
    /** True if file positions in the data file take up 5 bytes rather than 4. */
    @property (nonatomic) bool largeFile;
    /** True if the driving side (rule of the road) is known. */
    @property (nonatomic) bool drivingSideKnown;
    /** True if the driving side is known and the rule is to drive on the left. */
    @property (nonatomic) bool driveOnLeft;
    
    /** Initializes with a C++ metadata object: for internal use only. */
    -(id)initWithMetaData:(void*)aMetaData;
    /** Returns a pointer to the C++ metadata object: for internal use only. **/
    -(const void*)metaData;
@end

/**
Parameters used when inserting a map object into a writable map.
After creating the parameter object, add points and optionally set other parameters,
then call one of the functions which insert objects.
*/
@interface CartoTypeMapObjectParam: NSObject
    /** Initializes the parameter object with an object type, layer name and the coordinate type to be used when adding points. */
    -(id)initWithType:(CartoTypeMapObjectType)aType andLayer:(NSString*)aLayer andCoordType:(CartoTypeCoordType)aCoordType;
    /** Destroys the parameter object. */
    -(void)dealloc;
    /** Adds a new point to the object's geometry. */
    -(void)appendX:(double)aX andY:(double)aY;
    /** Adds a new point to the object's geometry, specifying the point type as well as the coordinates. */
    -(void)appendX:(double)aX andY:(double)aY andPointType:(CartoTypePointType)aPointType;
    /** Starts a new contour; use this if the object is made of multiple contours. */
    -(void)beginContour;
    /** (for internal use) Gets a pointer to the underlying C++ geometry object. */
    -(void*)geometry;

    /** The map handle: 0 can be used for the writable in-memory map. */
    @property (nonatomic) uint32_t mapHandle;
    /** The name of the layer. */
    @property (nonatomic, strong) NSString* layer;
    /** The radius (default = 0); if the radius is greater than zero, a point object is turned into a circle with the given radius, and other objects are made into envelopes of that size. */
    @property (nonatomic) double radius;
    /** The X radius (default = 0); if x and y radii are greater than zero, a point object is turned into an ellipse with the given radii. */
    @property (nonatomic) double radiusX;
    /** The Y radius (default = 0); if x and y radii are greater than zero, a point object is turned into an ellipse with the given radii. */
    @property (nonatomic) double radiusY;
    /** The coordinate type of the radii (default = MapMeterCoordType). */
    @property (nonatomic) CartoTypeCoordType radiusCoordType;
    /** The rotation in degrees applied to an ellipse map object. */
    @property (nonatomic) double rotationDegrees;
    /** Integer attribute value (default = 0). */
    @property (nonatomic, strong) CartoTypeFeatureInfo* featureInfo;
    /** String attributes if any (default = null); if there are no string attributes this value map be null. */
    @property (nonatomic, strong) NSString* stringAttrib;
    /** The object identifier (default = 0); if it is zero an identifier is assigned and returned in this property */
    @property (nonatomic) uint64_t objectId;
    /** If true (the default) replace the object if its ID is a duplicate of that of an existing object. */
    @property (nonatomic) bool replace;
@end

#endif
